@@ -34,6 +34,7 @@ import attr
 
 from hypothesis.errors import InvalidArgument, HypothesisDeprecationWarning
 from hypothesis.configuration import hypothesis_home_dir
+from hypothesis.internal.compat import PYPY
 from hypothesis.utils.conventions import UniqueIdentifier, not_set
 from hypothesis.utils.dynamicvariables import DynamicVariable
 
@@ -265,8 +266,8 @@ class settings(settingsMeta('settings', (object,), {})):
         If this was explicitly set at settings instantiation then that
         value will be used (even if it was None). If not and the
         database_file setting is not None this will be lazily loaded as
-        an ExampleDatabase using that file the first time this property
-        is accessed on a particular thread.
+        an ExampleDatabase, using that file the first time that this
+        property is accessed on a particular thread.
 
         """
         if self._database is not_set and self.database_file is not None:
@@ -355,7 +356,7 @@ search space.
 
 settings.define_setting(
     'max_examples',
-    default=200,
+    default=100,
     description="""
 Once this many satisfying examples have been considered without finding any
 counter-example, falsification will terminate.
@@ -377,7 +378,7 @@ settings.define_setting(
     default=10,
     description="""
 Hypothesis will try this many variations on a single example before moving on
-to an entirely fresh start. If you've got hard to satisfy properties raising
+to an entirely fresh start. If you've got hard to satisfy properties, raising
 this might help, but you probably shouldn't touch this dial unless you really
 know what you're doing.
 """
@@ -478,7 +479,7 @@ settings.define_setting(
     ),
     show_default=False,
     description="""
-    database: An instance of hypothesis.database.ExampleDatabase that will be
+    An instance of hypothesis.database.ExampleDatabase that will be
 used to save examples to and load previous examples from. May be None
 in which case no storage will be used.
 """
@@ -615,7 +616,7 @@ attempting to actually execute your test.
 settings.define_setting(
     'suppress_health_check',
     default=[],
-    description="""A list of health checks to disable"""
+    description="""A list of health checks to disable."""
 )
 
 settings.define_setting(
@@ -632,9 +633,13 @@ settings.define_setting(
     default=not_set,
     description=u"""
 If set, a time in milliseconds (which may be a float to express
-smaller units of time) that a test is not allowed to exceed. Tests which take
-longer than that will be converted into errors. Set this to None to disable
-this behaviour entirely.
+smaller units of time) that each individual example (i.e. each time your test
+function is called, not the whole decorated test) within a test is not
+allowed to exceed. Tests which take longer than that may be converted into
+errors (but will not necessarily be if close to the deadline, to allow some
+variability in test run time).
+
+Set this to None to disable this behaviour entirely.
 
 In future this will default to 200. For now, a
 HypothesisDeprecationWarning will be emitted if you exceed that default
@@ -642,6 +647,20 @@ deadline and have not explicitly set a deadline yourself.
 """
 )
 
+settings.define_setting(
+    'use_coverage',
+    default=not PYPY,
+    show_default=False,
+    description="""
+Whether to use coverage information to improve Hypothesis's ability to find
+bugs. You should generally leave this turned on unless your code performs
+poorly when run under coverage.
+
+Note: This is turned on by default except on pypy, where coverage performance
+is sufficiently poor as to make this unusable.
+"""
+)
+
 settings.lock_further_definitions()
 
 settings.register_profile('default', settings())
