@@ -17,9 +17,8 @@
 
 from __future__ import division, print_function, absolute_import
 
-from collections import namedtuple
-
 import hypothesis.internal.conjecture.utils as cu
+from hypothesis.errors import InvalidArgument
 from hypothesis.internal.compat import OrderedDict, hbytes
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
     MappedSearchStrategy, one_of_strategies
@@ -28,11 +27,7 @@ from hypothesis.searchstrategy.strategies import SearchStrategy, \
 class TupleStrategy(SearchStrategy):
 
     """A strategy responsible for fixed length tuples based on heterogenous
-    strategies for each of their elements.
-
-    This also handles namedtuples
-
-    """
+    strategies for each of their elements."""
 
     def __init__(self,
                  strategies, tuple_type):
@@ -53,6 +48,9 @@ class TupleStrategy(SearchStrategy):
             tuple_string,
         )
 
+    def calc_has_reusable_values(self, recur):
+        return all(recur(e) for e in self.element_strategies)
+
     def newtuple(self, xs):
         """Produce a new tuple of the correct type."""
         return tuple(xs)
@@ -62,8 +60,8 @@ class TupleStrategy(SearchStrategy):
             data.draw(e) for e in self.element_strategies
         )
 
-    def calc_is_empty(self):
-        return any(e.is_empty for e in self.element_strategies)
+    def calc_is_empty(self, recur):
+        return any(recur(e) for e in self.element_strategies)
 
 
 TERMINATOR = hbytes(b'\0')
@@ -96,8 +94,23 @@ class ListStrategy(SearchStrategy):
 
     def do_validate(self):
         self.element_strategy.validate()
+        if self.is_empty:
+            raise InvalidArgument((
+                'Cannot create non-empty lists with elements drawn from '
+                'strategy %r because it has no values.') % (
+                self.element_strategy,))
+
+    def calc_is_empty(self, recur):
+        if self.min_size == 0:
+            return False
+        else:
+            return recur(self.element_strategy)
 
     def do_draw(self, data):
+        if self.element_strategy.is_empty:
+            assert self.min_size == 0
+            return []
+
         elements = cu.many(
             data,
             min_size=self.min_size, max_size=self.max_size,
@@ -134,12 +147,23 @@ class UniqueListStrategy(SearchStrategy):
 
     def do_validate(self):
         self.element_strategy.validate()
+        if self.is_empty:
+            raise InvalidArgument((
+                'Cannot create non-empty lists with elements drawn from '
+                'strategy %r because it has no values.') % (
+                self.element_strategy,))
 
-    Parameter = namedtuple(
-        'Parameter', ('parameter_seed', 'parameter')
-    )
+    def calc_is_empty(self, recur):
+        if self.min_size == 0:
+            return False
+        else:
+            return recur(self.element_strategy)
 
     def do_draw(self, data):
+        if self.element_strategy.is_empty:
+            assert self.min_size == 0
+            return []
+
         elements = cu.many(
             data,
             min_size=self.min_size, max_size=self.max_size,
@@ -190,6 +214,9 @@ class FixedKeysDictStrategy(MappedSearchStrategy):
             )
         )
 
+    def calc_is_empty(self, recur):
+        return recur(self.mapped_strategy)
+
     def __repr__(self):
         return 'FixedKeysDictStrategy(%r, %r)' % (
             self.keys, self.mapped_strategy)
